"""Plugin helper module."""

from __future__ import annotations

from importlib.metadata import entry_points


def detect_plugins(group: str = "galactic", name: str | None = None) -> None:
    """
    Detect plugins.

    Detects plugins declared by their group and executes their entry point as a
    function.

    Parameters
    ----------
    group
        The plugin group
    name
        The plugin name

    """
    for entry in entry_points(group=group):
        if name is None or name == entry.name:
            entry.load()()



"""
Utilities for pip configuration.
"""

from __future__ import annotations

import os
import sys
from pathlib import Path
from typing import TYPE_CHECKING

import platformdirs

if TYPE_CHECKING:
    from collections.abc import Mapping


PIP_CONFIG_FILE_ENV_VAR = "PIP_CONFIG_FILE"
VIRTUAL_ENV_ENV_VAR = "VIRTUAL_ENV"
CONDA_PREFIX_ENV_VAR = "CONDA_PREFIX"
WINDOWS_PIP_CONFIG_FILENAME = "pip.ini"
UNIX_PIP_CONFIG_FILENAME = "pip.conf"


def pip_config_filename() -> str:
    """
    Return the pip configuration filename for the current platform.

    Returns
    -------
    str
        ``pip.ini`` on Windows, ``pip.conf`` elsewhere.

    """
    if sys.platform.startswith("win"):
        return WINDOWS_PIP_CONFIG_FILENAME
    return UNIX_PIP_CONFIG_FILENAME


def resolve_pip_config_path(*, env: Mapping[str, str] | None = None) -> Path:
    """
    Resolve the pip configuration file path.

    The resolution follows this priority order:

    1. ``PIP_CONFIG_FILE`` environment variable (explicit override).
    3. ``CONDA_PREFIX`` — path inside the active conda environment.
    2. ``VIRTUAL_ENV`` — path inside the active virtual environment.
    4. User-level config directory resolved by :mod:`platformdirs`.

    Parameters
    ----------
    env
        Environment variables mapping. Defaults to :data:`os.environ`.

    Returns
    -------
    Path
        The path to the pip configuration file.

    """
    environment: Mapping[str, str] = env if env is not None else os.environ
    filename = pip_config_filename()

    # Priority 1: explicit override
    if pip_config_file := environment.get(PIP_CONFIG_FILE_ENV_VAR):
        return Path(pip_config_file)

    # Priority 2: inside a conda environment
    if conda_prefix := environment.get(CONDA_PREFIX_ENV_VAR):
        return Path(conda_prefix) / filename

    # Priority 3: inside a virtual environment
    if virtual_env := environment.get(VIRTUAL_ENV_ENV_VAR):
        return Path(virtual_env) / filename

    # Priority 4: user-level config resolved by platformdirs
    return platformdirs.user_config_path("pip") / filename



"""
Defines three helper functions.

* :func:`detect_plugins` for detecting plugins
* :func:`default_repr` for providing a unified string representation of objects
* :func:`resolve_pip_config_path` for resolving the pip config file path
"""

from ._config import resolve_pip_config_path
from ._plugins import detect_plugins
from ._repr import default_repr

__all__ = ("detect_plugins", "default_repr", "resolve_pip_config_path")



"""The configuration module for sphinx."""

import json
import os

metadata = json.load(os.popen("hatch project metadata"))
project = metadata["name"]
author = ",".join(author["name"] for author in metadata["authors"])
version = metadata["version"]
project_copyright = f"2018-%Y, {author}"

# -- General configuration ------------------------------------------------

extensions = [
    "myst_parser",
    "sphinx.ext.autodoc",
    "sphinx.ext.autosummary",
    "sphinx.ext.todo",
    "sphinx.ext.coverage",
    "sphinx.ext.mathjax",
    "sphinx.ext.intersphinx",
    "sphinx.ext.napoleon",
    "sphinx_book_theme",
    "nbsphinx",
    "sphinxcontrib.plantuml",
    "sphinx_autodoc_typehints",
    "sphinx_codeautolink",
    "sphinx_copybutton",
]

copybutton_prompt_text = (
    r">>> "
    r"|\.\.\. "
    r"|(?:\([^)\n]*\)\s*)?\$ "
    r"|In \[\d*\]: "
    r"| {2,5}\.\.\.: "
    r"| {5,8}: "
)
copybutton_prompt_is_regexp = True
copybutton_line_continuation_character = "\\"
plantuml_output_format = "svg_img"

add_module_names = False

# nbsphinx settings
nbsphinx_allow_errors = True

# Napoleon settings
napoleon_google_docstring = True
napoleon_numpy_docstring = True
napoleon_include_init_with_doc = True
napoleon_include_private_with_doc = False
napoleon_include_special_with_doc = True
napoleon_use_admonition_for_examples = True
napoleon_use_admonition_for_notes = True
napoleon_use_admonition_for_references = True
napoleon_use_ivar = False
napoleon_use_param = True
napoleon_use_rtype = True

# intersphinx setting
intersphinx_mapping = {"python": ("https://docs.python.org/3", None)}

# autodoc and autosummary settings
autosummary_generate = True
autodoc_member_order = "bysource"
autodoc_default_options = {
    "member-order": "bysource",
    "members": True,
    "show-inheritance": True,
}

typehints_defaults = "comma"

# Add any paths that contain templates here, relative to this directory.
templates_path = ["_templates"]

# The suffix(es) of source filenames.
source_suffix = {".rst": "restructuredtext", ".md": "markdown"}

# The master toctree document.
master_doc = "index"

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#
# This is also used if you do content translation via gettext catalogs.
# Usually you set "language" from the command line for these cases.
language = "en"

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This patterns also effect to html_static_path and html_extra_path
exclude_patterns = ["_build", "**.ipynb_checkpoints"]

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = "sphinx"

# If true, `todo` and `todoList` produce output, else they produce nothing.
todo_include_todos = True

# -- Options for HTML output ----------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = "sphinx_book_theme"

# Theme options are theme-specific and customize the look and feel of a theme
# further.  For a list of options available for each theme, see the
# documentation.
html_theme_options = {
    "max_navbar_depth": 1,
    "logo": {
        "image_dark": "icon-dark.svg",
        "text": project[len("galactic-") :],
    },
    "use_download_button": False,
    "use_fullscreen_button": False,
    "switcher": {
        "version_match": "latest",
        "json_url": "/docs/helpers/core/versions.json",
    },
    "icon_links": [
        {
            "name": "GitLab",
            "url": "https://gitlab.univ-lr.fr/galactic/public/src/helpers/galactic-helper-core/",
            "icon": "fa-brands fa-gitlab",
            "type": "fontawesome",
        },
    ],
}

# Custom sidebar templates, must be a dictionary that maps document names
# to template names.
html_sidebars = {
    "**": [
        "navbar-logo.html",
        "version-switcher.html",
        "icon-links.html",
        "search-button-field.html",
        "sbt-sidebar-nav.html",
    ],
}
html_show_sourcelink = False
html_favicon = "thegalactic.ico"
html_logo = "icon-light.svg"
html_title = ""



from __future__ import annotations

import subprocess
from pathlib import Path

import tomllib
from hatchling.builders.hooks.plugin.interface import BuildHookInterface
from hatchling.metadata.plugin.interface import MetadataHookInterface


class CustomMetaDataHook(MetadataHookInterface):
    """
    Custom metadata hook.
    """

    def update(self, metadata: dict[str, object]) -> None:
        """
        Update the metadata for the store package.

        Parameters
        ----------
        metadata
            The current metadata
        """
        source_path = Path(self.root, "..", "pyproject.toml")
        if source_path.exists():
            with source_path.open("rb") as f:
                source_metadata = tomllib.load(f)
                project = source_metadata["project"]
            result = subprocess.run(
                ["hatch", "version"],
                cwd=Path(self.root, ".."),
                check=True,
                capture_output=True,
                text=True,
            )
            version = result.stdout.strip()
            # Set version
            metadata["version"] = version
            # Set metadata fields
            for key in (
                "name",
                "description",
                "requires-python",
                "license",
                "authors",
                "maintainers",
                "keywords",
                "classifiers",
                "urls",
                "gui-scripts",
                "scripts",
                "entry-points",
            ):
                if key in project:
                    metadata[key] = project[key]
            # Update name
            metadata["name"] += ".store"
            # Set dependencies
            if "dependencies" in project:
                metadata["dependencies"] = [f"{project['name']}=={version}"]
            # Set optional dependencies
            if "optional-dependencies" in project:
                metadata["optional-dependencies"] = {}
                for group in project["optional-dependencies"]:
                    metadata["optional-dependencies"][group] = [
                        f"{project['name']}[{group}]=={version}",
                    ]

            # Set readme if needed
            try:
                targets = source_metadata["tool"]["hatch"]["build"]["targets"]
                readme = targets["package.store"]["metadata"]["readme"]
                metadata["readme"] = readme
                internal = Path(readme)
                external = Path("..", readme)
                if not internal.exists():
                    internal.symlink_to(external)
            except KeyError:
                pass


class CustomBuildHook(BuildHookInterface):
    """
    Custom build hook.
    """

    def initialize(
        self,
        version: str,
        build_data: dict[str, object],
    ) -> None:
        """
        Initialize the build process.

        Parameters
        ----------
        version
            The version (``standard``, ``editable``, ...)
        build_data
            The build data

        """
        if version == "standard":
            source_path = Path(self.root, "..", "pyproject.toml")
            if source_path.exists():
                with source_path.open("rb") as f:
                    source_metadata = tomllib.load(f)
                    # Set readme if needed
                    try:
                        targets = source_metadata["tool"]["hatch"]["build"]["targets"]
                        readme = targets["package.store"]["metadata"]["readme"]
                        internal = Path(readme)
                        external = Path("..", readme)
                        if not internal.exists():
                            internal.symlink_to(external)
                    except KeyError:
                        pass

    def finalize(
        self,
        version: str,
        build_data: dict[str, object],
        artifact_path: str,
    ) -> None:
        """
        Finalize the build process.

        Parameters
        ----------
        version
            The version (``standard``, ``editable``, ...)
        build_data
            The build data
        artifact_path
            The artifact path

        """
        if version == "standard":
            source_path = Path(self.root, "..", "pyproject.toml")
            if source_path.exists():
                with source_path.open("rb") as f:
                    source_metadata = tomllib.load(f)
                    # Set readme if needed
                    try:
                        targets = source_metadata["tool"]["hatch"]["build"]["targets"]
                        readme = targets["package.store"]["metadata"]["readme"]
                        internal = Path(readme)
                        internal.unlink(missing_ok=True)
                    except KeyError:
                        pass



from collections.abc import Mapping
from pathlib import Path

def detect_plugins(group: str = "galactic", name: str | None = None) -> None: ...
def default_repr(
    item: object,
    *,
    class_name: str | None = None,
    module_name: str | None = None,
) -> str: ...
def resolve_pip_config_path(*, env: Mapping[str, str] | None = None) -> Path: ...



"""Module for representing objects."""


def default_repr(
    elem: object,
    *,
    class_name: str | None = None,
    module_name: str | None = None,
) -> str:
    """
    Generate a string representation of an object.

    This function generates a string representation of a given Python object.
    It is particularly useful for debugging or logging, where a clear and informative
    representation of an object is needed.

    The function takes an object, an optional class name and a optional module name as
    input. If the class name is provided, it is used as the class name in the
    representation. If not provided, the function uses the actual class name of the
    object. If the module name is provided, it is used as the module path in the
    representation. If not provided, the function constructs the module path by
    extracting the module name from the object's type and filtering out any private
    elements (those starting with an underscore).

    Parameters
    ----------
    elem
        The Python object to represent
    class_name
        An optional class name to use in the representation
    module_name
        An optional module name to use in the representation. If not provided,
        the module name is derived from the object's type. If provided and does not end
        with a dot, a dot is appended to it.

    Returns
    -------
    str
        A string representation of the object

    """
    class_name = class_name or type(elem).__name__
    if module_name is None:
        module_name = (
            ".".join(
                element
                for element in type(elem).__module__.split(".")
                if element[0] != "_"
            )
            + "."
        )
    elif module_name and module_name[-1] != ".":
        module_name += "."
    return f"<{module_name}{class_name} object at {id(elem):#x}>"



"""Custom build script."""

import subprocess
from pathlib import Path

from hatchling.builders.hooks.plugin.interface import BuildHookInterface
from hatchling.metadata.plugin.interface import MetadataHookInterface


class CustomMetaDataHook(MetadataHookInterface):
    """
    Metadata hook.
    """

    def update(self, metadata: dict[str, object]) -> None:
        """
        Update metadata.

        Parameters
        ----------
        metadata
            The current metadata

        """


class CustomBuildHook(BuildHookInterface):
    """
    Custom build hook.
    """

    def initialize(
        self,
        version: str,
        build_data: dict[str, object],
    ) -> None:
        """
        Initialize the build process.

        Parameters
        ----------
        version
            The version (``standard``, ``editable``, ...)
        build_data
            The build data

        """
        super().initialize(version, build_data)
        if version == "standard" and self.target_name == "wheel":
            # Build extras
            base_path = Path(self.root)
            print("⏳ Extra packages building")
            for folder in base_path.iterdir():
                if folder.is_dir() and folder.name.startswith("extra."):
                    # Build package
                    result = subprocess.run(
                        [
                            "uv",
                            "build",
                            folder,
                            "--out-dir",
                            Path(self.directory, "extras"),
                            "--wheel",
                        ],
                        capture_output=True,
                        check=True,
                        text=True,
                    )
                    extra = folder.name.split(".")[-1]
                    self.metadata.core.optional_dependencies[extra] = [
                        f"{self.metadata.name}.{extra}=={self.metadata.version}",
                    ]
            print("✅ Extra packages built")
            print("⏳ Store package building")
            # Build store
            store_path = Path(self.root, "package.store")
            if store_path.exists():
                # Build package
                result = subprocess.run(
                    [
                        "uv",
                        "build",
                        store_path,
                        "--out-dir",
                        Path(self.directory, "store"),
                        "--wheel",
                    ],
                    capture_output=True,
                    check=True,
                    text=True,
                )
            print("✅ Store package built")
